from .conversion_file import ConversionFile, ConversionEntry
from .helpers import BestRejection
from .resources import SirilResource
from .siril import SirilCli, SirilError

__all__ = ["ConversionFile", "ConversionEntry", "BestRejection", "SirilResource", "SirilCli", "SirilError"]



from __future__ import annotations
import typing as t

from dataclasses import dataclass
from .system import container_aware_cpu_limit, container_aware_memory_limit_gb


@dataclass
class SirilResource:
    """Represents the resources to use for Siril"""

    # How much CPU to use (in number of cores, default is all)
    cpu_limit: t.Optional[int] = None

    # How much memory to use (in GB, default is 90% of available memory)
    memory_limit: t.Optional[str] = None

    # How much of the available memory to use (0.9 = 90%, default is 90%)
    memory_percent: float = 0.9

    @staticmethod
    def container_aware_limits() -> SirilResource:
        """Get the limits for Siril in a container environment"""
        return SirilResource(
            cpu_limit=container_aware_cpu_limit(),
            memory_limit=container_aware_memory_limit_gb(),
        )

    @staticmethod
    def default_limits() -> SirilResource:
        """Get the default limits for Siril"""
        return SirilResource()



from __future__ import annotations

import structlog.stdlib
import asyncio
import cappa
import pathlib
import tempfile
import typing as t
from dataclasses import dataclass

from async_siril import SirilCli, BestRejection
from async_siril.command import (
    setext,
    set32bits,
    cd,
    convert,
    stack,
    register,
    seqsubsky,
    seqapplyreg,
    mirrorx,
    load,
    save,
)
from async_siril.command import fits_extension, stack_norm

log = structlog.stdlib.get_logger()


@dataclass
class CreateMasterLight:
    pp_folder: t.Annotated[pathlib.Path, cappa.Arg(help="Path to the raw folder of calibrated Light frames")]
    output: t.Annotated[pathlib.Path, cappa.Arg(help="Path to the output folder")]
    ext: t.Annotated[
        fits_extension, cappa.Arg(short=True, default=fits_extension.FITS_EXT_FIT, help="Extension of the Light frames")
    ]

    background_extraction: t.Annotated[bool, cappa.Arg(short=True, default=False, help="Enable background extraction")]
    name: t.Annotated[str, cappa.Arg(short=True, default="LIGHT_2025-06-30", help="Name of the master light")]

    async def __call__(self) -> None:
        log.info("Starting create master light")
        log.info(f"input: {self.pp_folder}")
        log.info(f"output: {self.output}")
        log.info(f"extension: {self.ext}")
        log.info(f"name: {self.name}")

        # Check if output folder exists else make it
        if not self.output.exists():
            self.output.mkdir(parents=True, exist_ok=True)

        with tempfile.TemporaryDirectory(dir=self.pp_folder) as tempdir:  # type: ignore
            temp = pathlib.Path(tempdir)
            log.info(f"temp dir: {temp}")

            # Find the best rejection method
            rejection = BestRejection.find(list(self.pp_folder.glob(f"*.{self.ext.value}")))

            async with SirilCli(directory=self.pp_folder) as siril:
                # Caution: these settings are saved between Siril sessions
                await siril.command(setext(self.ext))
                await siril.command(set32bits())

                # Manage the next prefix
                prefix = "light_"

                await siril.command(convert(prefix, output_dir=f"./{temp.name}"))
                await siril.command(cd(f"{temp.name}"))

                if self.background_extraction:
                    # Background extraction
                    await siril.command(seqsubsky(prefix))
                    prefix = f"bkg_{prefix}"

                # Register all the images
                await siril.command(register(prefix, two_pass=True))

                # and generate their transformed version
                await siril.command(seqapplyreg(prefix))
                prefix = f"r_{prefix}"

                # Stack the background extracted images
                await siril.command(
                    stack(
                        prefix,
                        norm=stack_norm.NORM_ADD_SCALE,
                        filter_included=True,
                        output_norm=True,
                        rgb_equalization=True,
                        rejection=rejection.method,
                        lower_rej=rejection.low_threshold,
                        higher_rej=rejection.high_threshold,
                        out="siril_result",
                    )
                )

                await siril.command(load("siril_result"))
                await siril.command(mirrorx())
                await siril.command(save(f"{self.output}/{self.name}_linear_stack"))

        log.info(f"Master light created and saved to: {self.output}")


def main() -> None:  # pragma: no cover
    try:
        asyncio.run(cappa.invoke_async(CreateMasterLight))
    except Exception as e:
        log.exception("Unhandled exception")
        raise cappa.Exit("There was an error while executing", code=-1) from e


if __name__ == "__main__":
    main()



import asyncio
import asyncio.subprocess
import structlog.stdlib
import os
import platform
import subprocess
import typing as t

from .command import BaseCommand, setcpu, set as siril_set, capabilities
from .command_types import SirilSetting
from .event import AsyncSirilEventConsumer, AsyncSirilCommandProducer
from .resources import SirilResource
from pathlib import Path


logger = structlog.stdlib.get_logger("async_siril")


class SirilError(Exception):
    """Base class for Siril errors and exceptions"""

    def __init__(self, cmd: str, message: str):
        self.command = cmd
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return f"SirilError from command: `{self.command}` error: `{self.message}`"


class SirilCli(object):
    """
    Main class for interacting with Siril using the async context manager pattern

    async with SirilCli() as siril:
        await siril.command("stack")
    """

    def __init__(
        self,
        siril_exe: str = "siril-cli",
        directory: t.Optional[Path] = None,
        resources: SirilResource = SirilResource.default_limits(),
    ):
        self._siril_exe = self._find_siril_cli(siril_exe)
        logger.info("Found Siril CLI executable: %s", self._siril_exe)

        self._cwd = directory
        self._resources = resources

        self._process: t.Optional[asyncio.subprocess.Process] = None
        self._consumer = AsyncSirilEventConsumer()
        self._producer = AsyncSirilCommandProducer()
        self._log_tasks = []

        # Get the version of the executable
        output = subprocess.Popen([self._siril_exe, "--version"], stdout=subprocess.PIPE)
        response, _ = output.communicate()
        self.version = response.decode().rstrip()
        logger.info("Found %s version: %s", self._siril_exe, self.version)

    async def _start(self):
        logger.debug("Initializing Siril CLI with Async Consumer & Producer")
        self._consumer.start()
        logger.debug("Siril CLI outpipe: %s", self._consumer.pipe_path)
        self._producer.start()
        logger.debug("Siril CLI inpipe: %s", self._producer.pipe_path)

        params = ["--pipe", "--inpipe", self._producer.pipe_path, "--outpipe", self._consumer.pipe_path]
        if self._cwd is not None:
            params.insert(0, "-d")
            params.insert(1, str(self._cwd))

        logger.info("Starting Siril CLI with params: %s", params)
        self._process = await asyncio.create_subprocess_exec(
            self._siril_exe,
            *params,
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE,
        )
        logger.info("Siril CLI process started")

        # Start logging tasks in background
        self._log_tasks = [
            asyncio.create_task(self._log_stream(self._process.stdout, "stdout")),
            asyncio.create_task(self._log_stream(self._process.stderr, "stderr")),
        ]

        # Start reading and become ready when the CLI says so
        await self._consumer.siril_ready
        logger.info("Siril CLI is now ready for startup commands")

        # Call this command as the first thing we do for all sessions
        from .command import requires

        # Still needed as the first command to be called
        await self.command(requires("0.99.10"))
        if self._resources.cpu_limit is not None:
            await self.command(setcpu(self._resources.cpu_limit))
        if self._resources.memory_limit is not None:
            await self.set(SirilSetting.MEM_MODE, "1")
            await self.set(SirilSetting.MEM_AMOUNT, self._resources.memory_limit)

        await self.set(SirilSetting.MEM_RATIO, str(self._resources.memory_percent))
        await self.command(capabilities())
        logger.info("AsyncSiril is ready for additional commands")

    async def _stop(self):
        logger.info("Stopping AsyncSiril process")
        try:
            # Kill process first to break pipe connections and unblock I/O
            if self._process and self._process.returncode is None:
                self._process.kill()
                await self._process.wait()
                logger.info("Siril CLI Process killed")

            # Now stop consumer and producer - they should exit naturally since pipes are broken
            await self._consumer.stop()
            await self._producer.stop()

            # Cancel and wait for log tasks
            for task in self._log_tasks:
                if not task.done():
                    task.cancel()

            try:
                await asyncio.wait_for(asyncio.gather(*self._log_tasks, return_exceptions=True), timeout=1.0)
            except asyncio.TimeoutError:
                logger.warning("Log tasks did not cancel in time")
            logger.info("AsyncSiril Cleanup completed")
        except Exception as e:
            logger.error("error during close: %s" % e)

    async def _log_stream(self, stream, stream_name):
        """Read lines from a subprocess stream and log them via structlog."""
        try:
            while True:
                line = await stream.readline()
                if not line:
                    break
                decoded = line.decode().rstrip()
                logger.info("siril_output", stream=stream_name, message=decoded)
        except asyncio.CancelledError:
            logger.debug(f"Log stream {stream_name} cancelled")
            raise

    async def command(self, cmd: t.Union[str, t.List[str], BaseCommand, t.List[BaseCommand]]):
        """Will run a command on the Siril pipe and throw `SirilError`'s as it sees them."""

        def is_list_of_types(lst, _type):
            if lst and isinstance(lst, list):
                return all(isinstance(elem, _type) for elem in lst)
            else:
                return False

        if isinstance(cmd, str) or isinstance(cmd, BaseCommand):
            await self._run_command(str(cmd))
        elif is_list_of_types(cmd, str) or is_list_of_types(cmd, BaseCommand):
            for c in cmd:
                await self._run_command(str(c))
        else:
            logger.error("incorrect command type")

    async def failable_command(self, cmd: t.Union[str, BaseCommand]) -> bool:
        """Will run a command on the Siril pipe and return a bool result if successful (catching any errors)"""
        try:
            await self.command(cmd)
            return True
        except SirilError as siril_error:
            logger.warn(f"Error caught by failable_command: {str(siril_error)}")
            return False

    async def _run_command(self, _command: str):
        # Use the special command to close the wrapper
        if _command == "exit":
            await self.stop()
            return

        logger.info(f"running command: '{_command}'")

        # Write the command first
        await self._producer.send(_command)

        # Read it the events off the listening queue
        while True:
            result = await self._consumer.queue.get()
            self._consumer.queue.task_done()

            if result.errored:
                logger.info("result errored")
                raise SirilError(_command, result.message)

            if result.completed:
                logger.info("result completed")
                break

            if result.siril_ready:
                logger.info("siril ready")
                break
        logger.info("Command completed")

    async def set(self, key: SirilSetting, value: str | bool):
        """Set a Siril setting using the `set` command"""
        await self.command(siril_set(key=key, value=str(value).lower()))

    async def __aenter__(self):
        await self.start()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        from .command import exit

        await self.command(exit())

    async def start(self):
        """Manually start the Siril process and pipes"""
        await self._start()
        logger.info("SirilCli started")

    async def stop(self):
        """Manually stop the Siril process and pipes"""
        await self._stop()
        logger.info("SirilCli stopped")

    def _find_siril_cli(self, siril_exe: str = "siril-cli") -> str:
        """Find the path to the Siril CLI executable"""
        if os.path.exists(siril_exe):
            return siril_exe

        # Try to find Siril dynamically
        system = platform.system()
        possible_paths = []
        if system == "Windows":
            possible_paths.append("C:/msys64/mingw64/bin/siril-cli.exe")  # msys2 path when building locally
            possible_paths.append("C:/Program Files/SiriL/bin/siril-cli.exe")
        elif system == "Darwin":
            possible_paths.append("/Applications/Siril.app/Contents/MacOS/siril-cli")
            possible_paths.append("/Applications/Siril.app/Contents/MacOS/Siril")
        elif system == "Linux":
            possible_paths.append("/usr/local/bin/siril-cli")
            possible_paths.append("/usr/bin/siril-cli")

        for path in possible_paths:
            if os.path.exists(path):
                return path

        raise FileNotFoundError("Siril CLI executable not found")



from __future__ import annotations

import pathlib
import typing as t

from dataclasses import dataclass
from async_siril.command_types import stack_rejection


@dataclass
class BestRejection:
    method: stack_rejection
    low_threshold: float
    high_threshold: float

    @staticmethod
    def find(images: t.List[pathlib.Path]) -> BestRejection:
        """Find the best rejection method for a given number of images"""
        if len(images) == 0:
            raise ValueError("At least 1 image is required to find the best rejection")

        if len(images) <= 6:
            return BestRejection(
                method=stack_rejection.REJECTION_PERCENTILE,
                low_threshold=0.2,
                high_threshold=0.1,
            )
        elif 6 < len(images) <= 30:
            return BestRejection(
                method=stack_rejection.REJECTION_WINSORIZED,
                low_threshold=3,
                high_threshold=3,
            )
        else:
            return BestRejection(
                method=stack_rejection.REJECTION_LINEAR_FIT,
                low_threshold=5,
                high_threshold=5,
            )



from __future__ import annotations

import structlog.stdlib
import asyncio
import cappa
import pathlib
import tempfile
import typing as t
from dataclasses import dataclass

from async_siril import SirilCli
from async_siril.command import setext, set32bits, cd, convert, stack
from async_siril.command import fits_extension

log = structlog.stdlib.get_logger()


@dataclass
class CreateMasterBias:
    raw_folder: t.Annotated[pathlib.Path, cappa.Arg(help="Path to the raw folder of Bias frames")]
    ext: t.Annotated[
        fits_extension, cappa.Arg(short=True, default=fits_extension.FITS_EXT_FIT, help="Extension of the Bias frames")
    ]
    name: t.Annotated[str, cappa.Arg(short=True, default="BIAS_2025-06-30", help="Name of the master bias")]

    async def __call__(self) -> None:
        log.info("Starting create master bias")
        log.info(f"input: {self.raw_folder}")
        log.info(f"extension: {self.ext}")
        log.info(f"name: {self.name}")

        with tempfile.TemporaryDirectory(dir=self.raw_folder) as tempdir:  # type: ignore
            temp = pathlib.Path(tempdir)
            log.info(f"temp dir: {temp}")

            async with SirilCli(directory=self.raw_folder) as siril:
                # Caution: these settings are saved between Siril sessions
                await siril.command(setext(self.ext))
                await siril.command(set32bits())

                await siril.command(convert(self.name, output_dir=f"./{temp.name}"))
                await siril.command(cd(f"{temp.name}"))
                await siril.command(stack(self.name, out=f"../../{self.name}_stacked"))

        log.info("Master bias created")


def main() -> None:  # pragma: no cover
    try:
        asyncio.run(cappa.invoke_async(CreateMasterBias))
    except Exception as e:
        log.exception("Unhandled exception")
        raise cappa.Exit("There was an error while executing", code=-1) from e


if __name__ == "__main__":
    main()



import structlog.stdlib
import os
import os.path
import psutil
import typing as t
from psutil._common import bytes2human

logger = structlog.stdlib.get_logger("async_siril.system")


def human_readable_byte_size(num):
    """
    returns a human-readable representation of a raw number of bytes

    param num: how nay bytes ?
    :return: a human-readable representation of given bytes
    :rtype: str
    """
    for unit in ["", "Ki", "Mi", "Gi", "Ti", "Pi", "Ei", "Zi"]:
        if abs(num) < 1024.0:
            return "%3.3f %sB" % (num, unit)
        num /= 1024.0
    return "%.3f %sB" % (num, "Yi")


def available_memory():
    return psutil.virtual_memory().available


def memory_used():
    return bytes2human(psutil.Process().memory_info().rss)


def process_info():
    p = psutil.Process()
    info = {
        "pid": p.pid,
        "name": p.name(),
        "status": p.status(),
        "rss": bytes2human(psutil.Process().memory_info().rss),
    }
    return info


def container_aware_memory_limit_gb() -> t.Optional[str]:
    memory_limit_paths = [
        "/sys/fs/cgroup/memory/memory.limit_in_bytes",
        "/sys/fs/cgroup/memory/memory.low",
        "/sys/fs/cgroup/memory/memory.high",
        "/sys/fs/cgroup/memory/memory.max",
        "/sys/fs/cgroup/memory.low",
        "/sys/fs/cgroup/memory.high",
        "/sys/fs/cgroup/memory.max",
    ]

    for _file in memory_limit_paths:
        raw_result = read_int(_file)
        if raw_result is not None and raw_result > 0:
            logger.debug(f"found non-zero value in: {_file} - {raw_result}")
            gb_value = raw_result / 1024 / 1024 / 1024
            return "{:.2f}".format(gb_value)
    return None


def container_aware_cpu_limit() -> t.Optional[int]:
    # cgroup v1
    cgroup1_period_file = "/sys/fs/cgroup/cpu/cpu.cfs_period_us"
    cgroup1_quota_file = "/sys/fs/cgroup/cpu/cpu.cfs_quota_us"  # k8s cpu limit
    if os.path.exists(cgroup1_quota_file) and os.path.exists(cgroup1_period_file):
        logger.debug("found cgroup1 CPU limit")
        cpu_quota_us = read_int(cgroup1_quota_file)
        cpu_period_us = read_int(cgroup1_period_file)
        if cpu_quota_us is not None and cpu_period_us is not None:
            logger.debug(f"found cgroup1 CPU limit - quota: {cpu_quota_us}, period: {cpu_period_us}")
            return int(cpu_quota_us / cpu_period_us)
        else:
            logger.debug("no valid cgroup1 quota or period found")

    # cgroup v2
    cgroup2_file = "/sys/fs/cgroup/cpu.max"
    if os.path.exists(cgroup2_file):
        logger.debug("found cgroup1 memory limit")
        with open(cgroup2_file) as f:
            try:
                combined = f.readline()
                values = combined.split(" ")
                if len(values) >= 2:
                    cpu_quota_us = int(values[0])
                    cpu_period_us = int(values[1])
                    return int(cpu_quota_us / cpu_period_us)
            except ValueError as e:  # noqa: F841
                logger.debug("no valid cgroup2 quota or period found")
                pass
    return None


def read_int(_file) -> t.Optional[int]:
    if os.path.exists(_file):
        with open(_file) as f:
            try:
                return int(f.readline())
            except ValueError as e:  # noqa: F841
                pass
    return None



from __future__ import annotations

from enum import Enum
from dataclasses import dataclass


class SirilSetting(Enum):
    """Represents some of the common Siril settings, use `get -a` to discover more."""

    EXTENSION = "core.extension"
    FORCE_16BIT = "core.force_16bit"
    MEM_MODE = "core.mem_mode"
    MEM_AMOUNT = "core.mem_amount"
    MEM_RATIO = "core.mem_ratio"


@dataclass
class Rect:
    x: int
    y: int
    width: int
    height: int

    def __str__(self):
        return f"{self.x} {self.y} {self.width} {self.height}"


@dataclass
class SigmaRange:
    low: float
    high: float

    def __str__(self):
        return f"{self.low} {self.high}"


class clipmode(Enum):
    CLIPMODE_CLIP = "clip"
    CLIPMODE_RESCALE = "rescale"
    CLIPMODE_RGB_BLEND = "rgbblend"
    CLIPMODE_GLOBAL_RESCALE = "globalrescale"


class fits_extension(Enum):
    FITS_EXT_FIT = "fit"
    FITS_EXT_FITS = "fits"
    FITS_EXT_FTS = "fts"


class registration_transformation(Enum):
    REG_TRANSF_SHIFT = "shift"
    REG_TRANSF_SIMILARITY = "similarity"
    REG_TRANSF_AFFINE = "affine"
    REG_TRANSF_HOMOGRAPHY = "homography"


class pixel_interpolation(Enum):
    INTERP_NONE = "none"
    INTERP_NEAREST = "nearest"
    INTERP_CUBIC = "cubic"
    INTERP_LANCZOS4 = "lanczos4"
    INTERP_LINEAR = "linear"
    INTERP_AREA = "area"


class sequence_framing(Enum):
    FRAME_CURRENT = "current"
    FRAME_MIN = "min"
    FRAME_MAX = "max"
    FRAME_COG = "cog"


class sequence_filter_type(Enum):
    FILTER_NONE = ""
    FILTER_FWHM = "filter-fwhm"
    FILTER_WFWHM = "filter-wfwhm"
    FILTER_ROUNDNESS = "filter-round"
    FILTER_QUALITY = "filter-quality"
    FILTER_INCLUSION = "filter-incl"
    FILTER_BACKGROUND = "filter-bkg"
    FILTER_STAR_COUNT = "filter-nbstars"


class compression_type(Enum):
    COMPRESSION_RICE = "rice"
    COMPRESSION_GZIP1 = "gzip1"
    COMPRESSION_GZIP2 = "gzip2"


class stack_type(Enum):
    STACK_SUM = "sum"
    STACK_REJ = "rej"
    STACK_MED = "med"
    STACK_MIN = "min"
    STACK_MAX = "max"


class stack_norm(Enum):
    NO_NORM = "-nonorm"
    NORM_ADD = "-norm=add"
    NORM_ADD_SCALE = "-norm=addscale"
    NORM_MUL = "-norm=mul"
    NORM_MUL_SCALE = "-norm=mulscale"


class stack_rejection(Enum):
    REJECTION_NONE = "n"
    REJECTION_PERCENTILE = "p"
    REJECTION_SIGMA = "s"
    REJECTION_MEDIAN = "m"
    REJECTION_WINSORIZED = "w"
    REJECTION_LINEAR_FIT = "l"
    REJECTION_GESDT = "g"
    REJECTION_MAD = "a"


class stack_weighting(Enum):
    NO_WEIGHT = ""
    WEIGHT_FROM_NOISE = "-weight_from_noise"
    WEIGHT_FROM_WFWHM = "-weight_from_wfwhm"
    WEIGHT_FROM_NBSTARS = "-weight_from_nbstars"
    WEIGHT_FROM_NBSTACK = "-weight_from_nbstack"


class stack_rejmaps(Enum):
    NO_REJECTION_MAPS = ""
    TWO_REJECTION_MAPS = "-rejmaps"
    MERGED_REJECTION_MAPS = "-rejmap"


class magnitude_option(Enum):
    DEFAULT_MAGNITUDE = 1
    MAGNITUDE_OFFSET = 2
    ABSOLUTE_MAGNITUDE = 3


class star_catalog(Enum):
    TYCHO2 = "tycho2"
    NOMAD = "nomad"
    GAIA = "gaia"
    PPMXL = "ppmxl"
    BRIGHTSTARS = "brightstars"
    APASS = "apass"


class rmgreen_protection(Enum):
    AVERAGE_NEUTRAL = 0
    MAXIMUM_NEUTRAL = 1
    MAXIMUM_MASK = 2
    ADDITIVE_MASK = 3


class saturation_hue_range(Enum):
    PINK_ORANGE = 0
    ORANGE_YELLOW = 1
    YELLOW_CYAN = 2
    CYAN = 3
    CYAN_MAGENTA = 4
    MAGENTA_PINK = 5
    ALL = 6


class catalog_option(Enum):
    APASS = "apass"
    LOCAL_GAIA = "localgaia"
    GAIA = "gaia"


class online_catalog(Enum):
    TYCHO2 = "tycho2"
    NOMAD = "nomad"
    GAIA = "gaia"
    LOCAL_GAIA = "localgaia"
    PPMXL = "ppmxl"
    BSC = "bsc"
    APASS = "apass"
    GCVS = "gcvsc"
    VSX = "vsx"
    SIMBAD = "simbad"
    AAVSO_VSP = "aavso_chart"
    EXOPLANET_ARCHIVE = "exo"
    PGC = "pgc"
    SOLSYS = "solsys"


class extract_resample(Enum):
    HA = "ha"
    OIII = "oiii"


class ght_weighting(Enum):
    HUMAN = "human"
    EVEN = "even"
    INDEPENDENT = "independent"
    SATURATION = "sat"


class Channel(Enum):
    RED = 0
    GREEN = 1
    BLUE = 2


class channel_label(Enum):
    RED = "R"
    GREEN = "G"
    BLUE = "B"
    RED_GREEN = "RG"
    RED_BLUE = "RB"
    GREEN_BLUE = "GB"
    ALL = "RGB"


class limit_option(Enum):
    CLIP = "clip"
    POS_RESCALE = "posrescale"
    RESCALE = "rescale"


class wavelet_type(Enum):
    LINEAR = 1
    BSPLINE = 2


class star_range(Enum):
    NARROW = "narrow"
    WIDE = "wide"


class find_star_catalog(Enum):
    NOMAD = "nomad"
    APASS = "apass"


class drizzle_kernel(Enum):
    POINT = "point"
    TURBO = "turbo"
    SQUARE = "square"
    GAUSSIAN = "gaussian"
    LANCZOS2 = "lanczos2"
    LANCZOS3 = "lanczos3"


class split_option(Enum):
    HSL = "hsl"
    HSV = "hsv"
    CIELAB = "lab"


class spcc_list_type(Enum):
    OSCSENSOR = "oscsensor"
    MONOSENSOR = "monosensor"
    REDFILTER = "redfilter"
    GREENFILTER = "greenfilter"
    BLUEFILTER = "bluefilter"
    OSCFILTER = "oscfilter"
    OSCLPF = "osclpf"
    WHITEREF = "whiteref"


class stat_detail(Enum):
    BASIC = "basic"
    MAIN = "main"
    FULL = "full"


class psf_method(Enum):
    CLEAR = "clear"
    LOAD = "load"
    SAVE = "save"
    BLIND = "blind"
    STARS = "stars"
    MANUAL = "manual"


class manual_psf_method(Enum):
    GAUSSIAN = "gaussian"
    MOFFAT = "moffat"
    DISC = "disc"
    AIRY = "airy"



import re
import pathlib
import typing as t

from dataclasses import dataclass


@dataclass
class ConversionEntry:
    """
    Represents a single entry in a conversion file
    """

    original_file: pathlib.Path
    converted_file: pathlib.Path


@dataclass
class ConversionFile:
    """
    Represents a conversion file from siril as a result of the `convert` command or other sequence operations
    """

    entries: t.List[ConversionEntry]
    file: pathlib.Path

    def __init__(self, file: pathlib.Path):
        self.file = file
        self.entries = []
        self.read()

    def read(self):
        """
        Reads the conversion file and populates the entries list
        """
        if not self.file.exists():
            return None

        regex = r"'(.*?)'.*?'(.*?)'"
        with open(self.file) as txt_file:
            raw = txt_file.read()
            matches = re.finditer(regex, raw, re.MULTILINE)
            for _match_num, match in enumerate(matches, start=1):
                orig_file = pathlib.Path(match.group(1))
                conv_file = pathlib.Path(match.group(2))

                self.entries.append(ConversionEntry(orig_file, conv_file))



from __future__ import annotations

import structlog.stdlib
import asyncio
import cappa
import pathlib
import tempfile
import typing as t
from dataclasses import dataclass

from async_siril import SirilCli, ConversionFile
from async_siril.command import setext, set32bits, cd, convert, calibrate
from async_siril.command import fits_extension


log = structlog.stdlib.get_logger()


@dataclass
class CalibrateLight:
    raw_folder: t.Annotated[pathlib.Path, cappa.Arg(help="Path to the raw folder of Light frames")]
    output: t.Annotated[pathlib.Path, cappa.Arg(help="Path to the output folder")]
    ext: t.Annotated[
        fits_extension, cappa.Arg(short=True, default=fits_extension.FITS_EXT_FIT, help="Extension of the Light frames")
    ]

    dark: t.Annotated[t.Optional[pathlib.Path], cappa.Arg(short=True, help="Path to the master dark")]
    flat: t.Annotated[t.Optional[pathlib.Path], cappa.Arg(short=True, help="Path to the master flat")]
    bias: t.Annotated[t.Optional[pathlib.Path], cappa.Arg(short=True, help="Path to the master bias")]

    async def __call__(self) -> None:
        log.info("Starting calibrate light")
        log.info(f"input: {self.raw_folder}")
        log.info(f"extension: {self.ext}")
        log.info(f"dark: {self.dark}")
        log.info(f"flat: {self.flat}")
        log.info(f"bias: {self.bias}")

        if self.dark is None and self.flat is None and self.bias is None:
            log.error("Dark, flat, or bias not specified")
            raise cappa.Exit()

        # Check if all raw frames are OSC
        all_color = await self._all_color_raw_frames()
        log.info(f"Raw images are OSC: {all_color}")

        # Check if output folder exists else make it
        if not self.output.exists():
            self.output.mkdir(parents=True, exist_ok=True)

        with tempfile.TemporaryDirectory(dir=self.raw_folder) as tempdir:  # type: ignore
            temp = pathlib.Path(tempdir)
            log.info(f"temp dir: {temp}")

            async with SirilCli(directory=self.raw_folder) as siril:
                # Caution: these settings are saved between Siril sessions
                await siril.command(setext(self.ext))
                await siril.command(set32bits())

                await siril.command(convert("light_", output_dir=f"./{temp.name}"))
                await siril.command(cd(f"{temp.name}"))

                await siril.command(
                    calibrate(
                        "light_",
                        bias=f"{str(self.bias)}" if self.bias is not None else None,
                        dark=f"{str(self.dark)}" if self.dark is not None else None,
                        flat=f"{str(self.flat)}" if self.flat is not None else None,
                        cfa=all_color,
                        debayer=all_color,
                        equalize_cfa=all_color,
                    )
                )

                conversion = ConversionFile(temp / "light_conversion.txt")
                log.info(f"conversion: {conversion.entries}")
                await self._move_converted_files(conversion, prefix="pp_")

        log.info("Light calibrated and saved to: {self.output}")

    async def _all_color_raw_frames(self) -> bool:
        from astropy.io import fits  # type: ignore
        from astropy.io.fits import PrimaryHDU  # type: ignore

        def is_color(file: pathlib.Path) -> bool:
            with fits.open(file, ignore_missing_simple=True) as hdu_list:
                primary_hdu = next(h for h in hdu_list if isinstance(h, PrimaryHDU))
                return "BAYERPAT" in primary_hdu.header and primary_hdu.header["BAYERPAT"] != ""

        raw_files = list(self.raw_folder.glob(f"*.{self.ext.value}"))
        result = all(is_color(raw_file) for raw_file in raw_files)
        return result

    async def _move_converted_files(self, conversion: ConversionFile, prefix: str) -> None:
        log.info(f"Moving converted files to {self.output}")
        for entry in conversion.entries:
            converted_file = conversion.file.parent.joinpath(f"{prefix}{entry.converted_file.name}")
            converted_file.rename(self.output / f"{prefix}{entry.original_file.name}")


def main() -> None:  # pragma: no cover
    try:
        asyncio.run(cappa.invoke_async(CalibrateLight))
    except Exception as e:
        log.exception("Unhandled exception")
        raise cappa.Exit("There was an error while executing", code=-1) from e


if __name__ == "__main__":
    main()
